﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace Magnetometer_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		Magnetometer mag = null;
		private ErrorEventBox errorBox;
		private DataGraphBox xGraphBox, yGraphBox, zGraphBox;

		double ambientMagneticField = 0.57142; //Calgary

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			xGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			yGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			zGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			mag = new Magnetometer();

			initMagFieldGraph();

			mag.Attach += magnetometer_attach;
			mag.Detach += magnetometer_detach;
			mag.Error += magnetometer_error;

			// This causes the handler to get called without Invoke
			FastEventHandler<MagnetometerMagneticFieldChangeEventArgs> magEventHandler = new FastEventHandler<MagnetometerMagneticFieldChangeEventArgs>();
			mag.MagneticFieldChange += magEventHandler.Handler;
			magEventHandler.Event += fieldStrength_change;

			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				mag.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				mag.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				mag.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				mag.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					mag.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					mag.IsLocal = true;

				mag.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			mag.Close();
		}


		void magnetometer_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			settingsBox.Visible = true;
			outputBox.Visible = true;
			magFieldGraphBox.Visible = true;

			Magnetometer attachedDevice = (Magnetometer)sender;

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "G";
				changeTriggerTrk.minTxt = attachedDevice.MinMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinMagneticFieldChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxMagneticFieldChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.MagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.MagneticFieldChangeTrigger;

			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}

			updateGraphDataInterval();

			guiUpdateTimer.Start();
		}

		void magnetometer_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			guiUpdateTimer.Stop();
			_mag[0] = 0;
			_mag[1] = 0;
			_mag[2] = 0;
			_timestamp = 0;

			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			magFieldGraphBox.Visible = false;
			calibrationStatusTxt.Visible = false;
		}

		void magnetometer_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation) {
				xGraphBox.addGap();
				yGraphBox.addGap();
				zGraphBox.addGap();
			}
		}

		double[] _mag = { 0, 0, 0 };
		double _timestamp = 0;
		private void guiUpdateTimer_Tick(object sender, EventArgs e) {
			xaxistxt.Text = _mag[0].ToString() + " G";
			yaxistxt.Text = _mag[1].ToString() + " G";
			zaxistxt.Text = _mag[2].ToString() + " G";
			timestamptxt.Text = _timestamp.ToString() + " ms";

			try {
				drawMagFieldGraph();
			} catch { }
		}

		void fieldStrength_change(object sender, Phidget22.Events.MagnetometerMagneticFieldChangeEventArgs e) {

			_mag[0] = e.MagneticField[0];
			_mag[1] = e.MagneticField[1];
			_mag[2] = e.MagneticField[2];
			_timestamp = e.Timestamp;

			xGraphBox.addData(e.MagneticField[0], null, e.Timestamp);
			yGraphBox.addData(e.MagneticField[1], null, e.Timestamp);
			zGraphBox.addData(e.MagneticField[2], null, e.Timestamp);
		}

		private void updateGraphDataInterval() {
			try {
				if (mag.MagneticFieldChangeTrigger == 0) {
					xGraphBox.DataInterval = mag.DataInterval;
					yGraphBox.DataInterval = mag.DataInterval;
					zGraphBox.DataInterval = mag.DataInterval;
				} else {
					xGraphBox.DataInterval = null;
					yGraphBox.DataInterval = null;
					zGraphBox.DataInterval = null;
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating graph data interval: " + ex.Message); }
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}

		private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				mag.DataInterval = (int)dataIntervalTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggerTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void setCompassParamsButton_Click(object sender, EventArgs e) {
			ParameterInputBox tib = new ParameterInputBox();
			DialogResult result = tib.ShowDialog();

			if (result == DialogResult.OK) {

				if (tib.parameters.Length == 0) {
					try {
						mag.ResetCorrectionParameters();
						mag.SaveCorrectionParameters();
						calibrationStatusTxt.Text = "Un-Calibrated";
						calibrationStatusTxt.ForeColor = Color.Red;
						calibrationStatusTxt.Visible = true;
					} catch (PhidgetException ex) { errorBox.addMessage("Error resetting calibration: " + ex.Message); }
					return;
				}

				string[] paramVals = tib.parameters.Split(new char[] { ',' });
				if (paramVals.Length == 13) {
					try {
						mag.SetCorrectionParameters(
							double.Parse(paramVals[0]),
							double.Parse(paramVals[1]),
							double.Parse(paramVals[2]),
							double.Parse(paramVals[3]),
							double.Parse(paramVals[4]),
							double.Parse(paramVals[5]),
							double.Parse(paramVals[6]),
							double.Parse(paramVals[7]),
							double.Parse(paramVals[8]),
							double.Parse(paramVals[9]),
							double.Parse(paramVals[10]),
							double.Parse(paramVals[11]),
							double.Parse(paramVals[12])
						);
						mag.SaveCorrectionParameters();
						ambientMagneticField = double.Parse(paramVals[0]);
						calibrationStatusTxt.Text = "Calibrated";
						calibrationStatusTxt.ForeColor = Color.Green;
						calibrationStatusTxt.Visible = true;
					} catch (PhidgetException ex) { errorBox.addMessage("Error calibrating: " + ex.Message); }
				}
			}
		}


		private void xGraphButton_Click(object sender, EventArgs e) {
			if (!xGraphBox.Visible)
				xGraphBox.Show();
		}
		private void yGraphButton_Click(object sender, EventArgs e) {
			if (!yGraphBox.Visible)
				yGraphBox.Show();
		}

		private void zGraphButton_Click(object sender, EventArgs e) {
			if (!zGraphBox.Visible)
				zGraphBox.Show();
		}

		private void dataIntervalTrk_SetClick(object sender, EventArgs e) {
			try {
				mag.DataInterval = (int)dataIntervalTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					mag.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

		#region graph
		private BufferedGraphicsContext magFieldGraphicsContext;
		private BufferedGraphics magFieldGraphicsBuffer;
		private Bitmap magFieldDrawingSurface;

		private Rectangle magFieldCircleRectangle, magFieldBoundsRectangle;
		private Pen magFieldXYAxisPen, magFieldCirclePen;

		private float magFieldCircleDiameter, magFieldCircleRadius;

		private void initMagFieldGraph() {

			//initialize our graphical representation of the compasseration data
			magFieldDrawingSurface = new Bitmap(magFieldView.ClientRectangle.Width, magFieldView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);

			magFieldBoundsRectangle = new Rectangle(0, 0, magFieldDrawingSurface.Width, magFieldDrawingSurface.Height);
			magFieldBoundsRectangle.Inflate(-1, -1);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			magFieldXYAxisPen = new Pen(Color.Black, 2);
			magFieldCirclePen = new Pen(Color.DarkBlue, 2);

			magFieldCircleDiameter = (float)(magFieldDrawingSurface.Width / 1.1);
			magFieldCircleRadius = magFieldCircleDiameter / 2;

			magFieldCircleRectangle = new Rectangle((int)Math.Round(xCenter - magFieldCircleRadius), (int)Math.Round(yCenter - magFieldCircleRadius), (int)Math.Round(magFieldCircleDiameter), (int)Math.Round(magFieldCircleDiameter));
		}

		private void drawMagFieldGraph() {
			magFieldGraphicsContext = new BufferedGraphicsContext();
			magFieldGraphicsBuffer = magFieldGraphicsContext.Allocate(Graphics.FromImage(magFieldDrawingSurface), magFieldBoundsRectangle);
			magFieldGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			magFieldGraphicsBuffer.Graphics.Clear(SystemColors.Control);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			float xOld = xCenter + (float)(mag.MagneticField[0]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);
			float yOld = yCenter + (float)(mag.MagneticField[1]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);

			magFieldGraphicsBuffer.Graphics.DrawLine(magFieldXYAxisPen, xCenter, yCenter, xOld, yOld);
			magFieldGraphicsBuffer.Graphics.DrawEllipse(magFieldCirclePen, magFieldCircleRectangle);

			double zOut = (mag.MagneticField[2]) * (float)(1 / ambientMagneticField);
			if (zOut > 0) {
				magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Red, 2),
					new Rectangle((int)xCenter - (int)(magFieldCircleRadius * zOut), (int)yCenter - (int)(magFieldCircleRadius * zOut),
					(int)(magFieldCircleDiameter * zOut), (int)(magFieldCircleDiameter * zOut)));
			} else {
				magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Green, 2),
					new Rectangle((int)xCenter - (int)(magFieldCircleRadius * -zOut), (int)yCenter - (int)(magFieldCircleRadius * -zOut),
					(int)(magFieldCircleDiameter * -zOut), (int)(magFieldCircleDiameter * -zOut)));
			}

			magFieldGraphicsBuffer.Render(magFieldView.CreateGraphics());

			magFieldGraphicsBuffer.Dispose();

			magFieldGraphicsContext.Dispose();
		}
		#endregion


	}
}
